%{
  (* File: parser.mly

      Copyright (C) 2005-

        Jane Street Holding, LLC
        Author: Markus Mottl
        email: mmottl@janestcapital.com
        WWW: http://www.janestcapital.com/ocaml

     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
     version 2 of the License, or (at your option) any later version.

     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.

     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *)

  (** Parser: Grammar Specification for Parsing S-expressions *)

  open Lexing

  let parse_failure () =
    let pos = symbol_start_pos () in
    let msg =
      Printf.sprintf "Sexplib.Parser.sexp: failed to parse line %d char %d"
        pos.pos_lnum (pos.pos_cnum - pos.pos_bol) in
    failwith msg
%}

%token <string> STRING
%token LPAREN RPAREN EOF

%start main
%type <Type.t list> main

%start sexp
%type <Type.t> sexp

%start rev_sexps
%type <Type.t list> rev_sexps

%start maybe_sexp
%type <Type.t option> maybe_sexp

%%

main
  : EOF { [] }
  | rev_sexps EOF { List.rev $1 }
  | error { parse_failure () }

sexp
  : STRING { Type.Atom $1 }
  | LPAREN RPAREN { Type.List [] }
  | LPAREN rev_sexps RPAREN { Type.List (List.rev $2) }
  | error { parse_failure () }

maybe_sexp
  : sexp { Some $1 }
  | EOF { None }

rev_sexps
  : sexp { [$1] }
  | rev_sexps sexp { $2 :: $1 }
